@@ -61,16 +61,14 @@ bool anychar( const std::function<bool( char )> &predicate, const std::string_vi
 
 int indexOf( const std::string &s, const char c )
 {
-   for( size_t i {}; i < s.length(); i++ )
-      if( s[i] == c )
-         return static_cast<int>(i);
-   return -1;
+   const auto p { s.find(c) };
+   return p == std::string::npos ? -1 : static_cast<int>( p );
 }
 
 void permutAssign( std::string &lhs, const std::string &rhs,
                    const std::vector<int> &writeIndices, const std::vector<int> &readIndices )
 {
-   for( int i = 0; i < (int) writeIndices.size(); i++ )
+   for( int i = 0; i < static_cast<int>( writeIndices.size() ); i++ )
    {
       lhs[writeIndices[i]] = rhs[readIndices[i]];
    }
@@ -100,29 +98,30 @@ std::string lowercase( const std::string_view s )
 bool sameTextInvariant( const std::string_view a,
                         const std::string_view b )
 {
-   const auto l = a.length();
    if( b.length() != a.length() ) return false;
+#if defined(_WIN32)
+   // This appears to be still faster with MSVC and icl.exe
+   const auto l = a.length();
    for( size_t i {}; i < l; i++ )
    {
-      if( tolower( a[i] ) != tolower( b[i] ) )
+      if( a[i] != b[i] && tolower( a[i] ) != tolower( b[i] ) )
          return false;
    }
    return true;
+#else
+   // Much faster with GCC and Clang (and new Intel compiler)
+   return std::equal(a.begin(), a.end(), b.begin(),
+      []( const unsigned char c1, const unsigned char c2) {
+         return c1 == c2 || tolower(c1) == tolower(c2);
+   });
+#endif
 }
 
 std::string_view trim( const std::string_view s )
 {
    if( s.empty() ) return {};
-   int firstNonBlank { -1 }, lastNonBlank {};
-   for( int i {}; i < static_cast<int>( s.length() ); i++ )
-   {
-      if( static_cast<unsigned char>( s[i] ) > 32 )
-      {
-         if( firstNonBlank == -1 ) firstNonBlank = i;
-         lastNonBlank = i;
-      }
-   }
-   if( firstNonBlank == -1 ) return {};
+   const auto firstNonBlank = s.find_first_not_of( " \t\n\r" );
+   const auto lastNonBlank = s.find_last_not_of( " \t\n\r" );
    return s.substr( firstNonBlank, lastNonBlank - firstNonBlank + 1 );
 }
 
@@ -145,7 +144,7 @@ bool sameTextPrefix( const std::string_view s, const std::string_view prefix )
 
 bool hasNonBlank( const std::string_view s )
 {
-   return std::any_of( s.begin(), s.end(), []( char c ) {
+   return std::any_of( s.begin(), s.end(), []( const char c ) {
       return !in( c, ' ', '\t', '\r', '\n' );
    } );
 }
@@ -153,16 +152,17 @@ bool hasNonBlank( const std::string_view s )
 std::string trim( const std::string &s )
 {
    if( s.empty() ) return s;
-   if( !hasNonBlank( s ) ) return ""s;
    const auto firstNonBlank = s.find_first_not_of( " \t\n\r" );
+   if(firstNonBlank == std::string::npos) return ""s;
    const auto lastNonBlank = s.find_last_not_of( " \t\n\r" );
-   return s.substr( firstNonBlank, ( lastNonBlank - firstNonBlank ) + 1 );
+   return s.substr( firstNonBlank, lastNonBlank - firstNonBlank + 1 );
 }
 
 std::string trimRight( const std::string &s )
 {
    if( s.empty() || !isblank( s.back() ) ) return s;
    const auto lastNonBlank = s.find_last_not_of( " \t" );
+   if(lastNonBlank == std::string::npos) return ""s;
    return s.substr( 0, lastNonBlank + 1 );
 }
 
@@ -189,12 +189,16 @@ void trimRight( const std::string &s, std::string &storage )
       return;
    }
    const auto ub = s.find_last_not_of( " \t" ) + 1;
-   //storage = s.substr(0, ub);
+   if(ub == std::string::npos)
+   {
+      storage.clear();
+      return;
+   }
    storage.replace( 0, ub, s, 0, ub );
    storage.resize( ub );
 }
 
-std::string trimZeroesRight( const std::string &s, char DecimalSep )
+std::string trimZeroesRight( const std::string &s, const char DecimalSep )
 {
    if( s.find( DecimalSep ) == std::string::npos ) return s;
    int i { static_cast<int>( s.length() ) - 1 };
@@ -205,7 +209,7 @@ std::string trimZeroesRight( const std::string &s, char DecimalSep )
 
 bool hasCharLt( const std::string_view s, int n )
 {
-   return anychar( [&n]( char c ) { return (int) c < n; }, s );
+   return anychar( [&n]( const char c ) { return static_cast<int>( c ) < n; }, s );
 }
 
 double round( const double n, const int ndigits )
@@ -213,34 +217,27 @@ double round( const double n, const int ndigits )
    return std::round( n * std::pow( 10, ndigits ) ) * pow( 10, -ndigits );
 }
 
-void replaceChar( char a, char b, std::string &s )
+void replaceChar( const char a, const char b, std::string &s )
 {
    if( a == b ) return;
-   std::replace_if(
-           s.begin(), s.end(), [a]( char i ) { return i == a; }, b );
-   /*for(char &i : s)
-            if(i == a) i = b;*/
+   std::replace_if(s.begin(), s.end(), [a]( const char i ) { return i == a; }, b );
 }
 
-std::string quoteWhitespace( const std::string &s,
-                             char quotechar )
+std::string quoteWhitespace( const std::string &s, char quotechar )
 {
    return s.find( ' ' ) != std::string::npos ? ""s + quotechar + s + quotechar : s;
 }
 
-int strCompare( const char *S1,
-                const char *S2,
-                bool caseInsensitive )
+int strCompare( const char *S1, const char *S2, const bool caseInsensitive )
 {
    if( S1[0] == '\0' || S2[0] == '\0' )
       return ( S1[0] != '\0' ? 1 : 0 ) - ( S2[0] != '\0' ? 1 : 0 );
-   size_t K;
-   for( K = 0; S1[K] != '\0' && S2[K] != '\0'; K++ )
+   for( size_t K = 0; S1[K] != '\0' && S2[K] != '\0'; K++ )
    {
-      int c1 = static_cast<unsigned char>( caseInsensitive ? toupper( S1[K] ) : S1[K] );
-      int c2 = static_cast<unsigned char>( caseInsensitive ? toupper( S2[K] ) : S2[K] );
-      int d = c1 - c2;
-      if( d ) return d;
+      const int c1 = static_cast<unsigned char>( caseInsensitive ? toupper( S1[K] ) : S1[K] );
+      const int c2 = static_cast<unsigned char>( caseInsensitive ? toupper( S2[K] ) : S2[K] );
+      if( const int d = c1 - c2 )
+         return d;
    }
    return static_cast<int>( std::strlen( S1 ) - std::strlen( S2 ) );
 }
@@ -253,8 +250,7 @@ int strCompare( const char *S1,
      **/
 
 // Convert C++ standard library string to Delphi short string
-int strConvCppToDelphi( const std::string_view s,
-                        char *delphistr )
+int strConvCppToDelphi( const std::string_view s, char *delphistr )
 {
    if( s.length() > std::numeric_limits<uint8_t>::max() )
    {
@@ -268,31 +264,24 @@ int strConvCppToDelphi( const std::string_view s,
    return 0;
 }
 
-std::vector<size_t> substrPositions( const std::string_view s, const std::string_view substr )
-{
-   std::vector<size_t> positions;
-   for( size_t p { s.find( substr ) }; p != std::string::npos; p = s.find( substr, p + substr.size() ) )
-      positions.push_back( p );
-   return positions;
-}
-
 std::string replaceSubstrs( const std::string_view s, const std::string_view substr, const std::string_view replacement )
 {
-   if( substr == replacement ) return std::string { s };
-   std::string out {};
-   const int ssl = static_cast<int>( substr.length() );
-   const auto positions = substrPositions( s, substr );
-   for( int i = 0; i < (int) s.length(); i++ )
+   if( substr.empty() || substr == replacement )
+      return std::string { s };
+
+   std::string res;
+   res.reserve( s.length() );
+
+   size_t last_pos {}, find_pos;
+
+   while( ( find_pos = s.find( substr, last_pos ) ) != std::string_view::npos )
    {
-      if( in<size_t>( i, positions ) )
-      {
-         out += replacement;
-         i += ssl - 1;
-         continue;
-      }
-      out += s[i];
+      res.append( s.data() + last_pos, find_pos - last_pos );
+      res.append( replacement );
+      last_pos = find_pos + substr.length();
    }
-   return out;
+   res.append( s.data() + last_pos, s.length() - last_pos );
+   return res;
 }
 
 // Mimicks Delphi System.Val, see:
@@ -306,7 +295,7 @@ void val( const std::string &s, double &num, int &code )
 // Mimicks Delphi System.Val, see:
 // https://docwiki.embarcadero.com/Libraries/Sydney/en/System.Val
 // https://www.delphibasics.co.uk/RTL.php?Name=Val
-void val(const char* s, int slen, double& num, int& code)
+void val(const char* s, const int slen, double& num, int& code)
 {
    rtl::p3io::P3_Val_dd(s, slen, &num, &code);
 }
@@ -342,7 +331,7 @@ std::string zeros( const int n )
    return repeatChar( n, '0' );
 }
 
-int lastOccurence( std::string_view s, char c )
+int lastOccurence( const std::string_view s, const char c )
 {
    for( int i = static_cast<int>( s.length() ) - 1; i >= 0; i-- )
       if( s[i] == c ) return i;
@@ -371,15 +360,15 @@ int strLenNoWhitespace( const std::string_view s )
    } ) );
 }
 
-char &getCharAtIndexOrAppend( std::string &s, int ix )
+char &getCharAtIndexOrAppend( std::string &s, const int ix )
 {
    const auto l = s.length();
-   assert( ix >= 0 && ix <= (int) l && "Index not in valid range" );
+   assert( ix >= 0 && ix <= static_cast<int>( l ) && "Index not in valid range" );
    if( static_cast<size_t>( ix ) == l ) s.push_back( '\0' );
    return s[ix];
 }
 
-bool strContains( const std::string_view s, char c )
+bool strContains( const std::string_view s, const char c )
 {
    return s.find( c ) != std::string::npos;
 }
@@ -387,25 +376,25 @@ bool strContains( const std::string_view s, char c )
 bool strContains( const std::string_view s, const std::initializer_list<char> &cs )
 {
    return std::any_of( std::cbegin( s ), std::cend( s ),
-                       [&cs]( char c ) { return std::find( cs.begin(), cs.end(), c ) != cs.end(); } );
+                       [&cs]( const char c ) { return std::find( cs.begin(), cs.end(), c ) != cs.end(); } );
 }
 
-bool excl_or( bool a, bool b )
+bool excl_or( const bool a, const bool b )
 {
    return ( a && !b ) || ( !a && b );
 }
 
 int posOfSubstr( const std::string_view sub, const std::string_view s )
 {
-   const auto pos = s.find( sub );
-   return pos == std::string::npos ? -1 : (int) pos;
+   const auto p = s.find( sub );
+   return p == std::string::npos ? -1 : static_cast<int>( p );
 }
 
-std::list<std::string> split( const std::string_view s, char sep )
+std::list<std::string> split( const std::string_view s, const char sep )
 {
    std::list<std::string> res;
    std::string cur;
-   for( char c: s )
+   for( const char c : s )
    {
       if( c != sep ) cur += c;
       else if( !cur.empty() )
@@ -467,12 +456,12 @@ void assertOrMsg( bool condition, const std::string &msg )
 }
 
 // same as std::string::substr but silent when offset > input size
-std::string_view substr( const std::string_view s, int offset, int len )
+std::string_view substr( const std::string_view s, const int offset, const int len )
 {
-   return ( s.empty() || offset > (int) s.size() - 1 ) ? std::string_view {} : s.substr( offset, len );
+   return s.empty() || offset > static_cast<int>( s.size() ) - 1 ? std::string_view {} : s.substr( offset, len );
 }
 
-std::string constructStr( int size, const std::function<char( int )> &charForIndex )
+std::string constructStr( const int size, const std::function<char( int )> &charForIndex )
 {
    std::string s;
    s.resize( size );
@@ -481,10 +470,10 @@ std::string constructStr( int size, const std::function<char( int )> &charForInd
    return s;
 }
 
-std::string join( char sep, const std::initializer_list<std::string> &parts )
+std::string join( const char sep, const std::initializer_list<std::string> &parts )
 {
-   const int len = std::accumulate( parts.begin(), parts.end(), (int) parts.size() - 1,
-                                    []( int acc, const std::string &s ) -> int { return acc + (int) s.length(); } );
+   const int len = std::accumulate( parts.begin(), parts.end(), static_cast<int>( parts.size() ) - 1,
+                                    []( const int acc, const std::string &s ) -> int { return acc + static_cast<int>( s.length() ); } );
    std::string res( len, sep );
    int i {};
    for( const std::string &part: parts )
@@ -518,7 +507,7 @@ bool ends_with( const std::string &s, const std::string &suffix )
    return true;
 }
 
-std::string quoteWhitespaceDir( const std::string &s, char sep, char quotechar )
+std::string quoteWhitespaceDir( const std::string &s, const char sep, const char quotechar )
 {
    if( !strContains( s, ' ' ) ) return s;
    std::string s2 {};
@@ -591,10 +580,10 @@ int strCompare( const std::string_view S1, const std::string_view S2, const bool
    if( L > S2.length() ) L = S2.length();
    for( size_t K {}; K < L; K++ )
    {
-      const int c1 = caseInsensitive ? toupper( S1[K] ) : S1[K];
-      const int c2 = caseInsensitive ? toupper( S2[K] ) : S2[K];
-      const int d = c1 - c2;
-      if( d ) return d;
+      const int c1 = static_cast<unsigned char>(caseInsensitive ? toupper( S1[K] ) : S1[K]);
+      const int c2 = static_cast<unsigned char>(caseInsensitive ? toupper( S2[K] ) : S2[K]);
+      if( const int d = c1 - c2 )
+         return d;
    }
    return static_cast<int>( S1.length() - S2.length() );
 }
@@ -611,8 +600,8 @@ std::string *StringBuffer::getStr()
 
 int StringBuffer::getBufferSize() const { return bufferSize; }
 
-BinaryDiffMismatch::BinaryDiffMismatch( uint64_t offset, uint8_t lhs, uint8_t rhs ) : offset( offset ), lhs( lhs ),
-                                                                                      rhs( rhs ) {}
+BinaryDiffMismatch::BinaryDiffMismatch( const uint64_t offset, const uint8_t lhs, const uint8_t rhs )
+   : offset( offset ), lhs( lhs ), rhs( rhs ) {}
 
 bool checkBOMOffset( const tBomIndic &potBOM, int &BOMOffset, std::string &msg )
 {
@@ -677,7 +666,7 @@ int64_t queryPeakRSS()
    PROCESS_MEMORY_COUNTERS info;
    if( !GetProcessMemoryInfo( GetCurrentProcess(), &info, sizeof( info ) ) )
       return 0;
-   return (int64_t) info.PeakWorkingSetSize;
+   return static_cast<int64_t>( info.PeakWorkingSetSize );
 #elif defined( __linux )
    std::ifstream ifs { "/proc/self/status" };
    if( !ifs.is_open() ) return 0;
@@ -700,7 +689,7 @@ int64_t queryPeakRSS()
 void copy_to_uppercase( const std::string &s, char *buf )
 {
    int j {};
-   for( char c: s )
+   for( const char c : s )
       buf[j++] = toupper( c );
    buf[j] = '\0';
 }
@@ -724,26 +713,41 @@ void trimLeft( std::string &s )
 {
    size_t i;
    for( i = 0; i < s.length(); i++ )
-      if( s[i] != ' ' ) break;
+      if( !std::isspace( s[i] ) )
+         break;
    s.erase( 0, i );
 }
 
+double frac( const double x )
+{
+   return x - std::trunc(x);
+}
+
 void getline( FILE *f, std::string &s )
 {
    constexpr int bsize {512};
    std::array<char, bsize> buf;
-   if(!std::fgets( buf.data(), bsize, f ) && std::ferror(f))
-      return;
-   s.assign( buf.data() );
+   s.clear();
+   while(std::fgets( buf.data(), bsize, f )  && !ferror( f ))
+   {
+      s += buf.data();
+      if(!s.empty() && s.back() == '\n')
+         break;
+   }
 }
 
 std::string getline( FILE *f )
 {
    constexpr int bsize {512};
    std::array<char, bsize> buf {};
-   if(!std::fgets( buf.data(), bsize, f ) && std::ferror(f))
-      return {};
-   return buf.data();
+   std::string s;
+   while(std::fgets( buf.data(), bsize, f ) && !ferror( f ))
+   {
+      s += buf.data();
+      if(!s.empty() && s.back() == '\n')
+         break;
+   }
+   return s;
 }
 
 std::string strInflateWidth( const int num, const int targetStrLen, const char inflateChar )
