@@ -1,4 +1,4 @@
-#include "system_p3.h"
+#include "system_p3.hpp"
 
 #if defined(_WIN32)
 #include <Windows.h>
@@ -13,7 +13,7 @@ void getdir(uint8_t d, std::string &s)
 #if defined(_WIN32)
    char Drive[3], DirBuf[MAX_PATH], SaveBuf[MAX_PATH];
    if (d) {
-      Drive[0] = 'A' + d - 1;
+      Drive[0] = static_cast<char>('A' + d - 1);
       Drive[1] = ':';
       Drive[2] = '\0';
       GetCurrentDirectoryA(sizeof(SaveBuf), SaveBuf);
@@ -26,7 +26,7 @@ void getdir(uint8_t d, std::string &s)
    char DirBuf[512];
    if( const char *p { getcwd( DirBuf, sizeof( DirBuf ) ) }; !p )
    {
-      // this is unlikely, and not handled well here: exceptions?
+      // this is unlikely and not handled well here: exceptions?
       s.clear();
       return;
    }
